/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Copyright (C) 2014 John Layt <jlayt@kde.org>
** Contact: https://www.qt.io/licensing/
**
** This file is part of the plugins of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qcupsprintersupport_p.h"

#include "qcupsprintengine_p.h"
#include "qppdprintdevice.h"
#include <private/qprinterinfo_p.h>
#include <private/qprintdevice_p.h>

#include <QtPrintSupport/QPrinterInfo>

#include <cups/ppd.h>
#ifndef QT_LINUXBASE // LSB merges everything into cups.h
# include <cups/language.h>
#endif

typedef struct
{
    cups_dest_t *printers;
    int num_printers;
} EnumDestsContext;

static int enum_dest_cb2 (void *user_data, unsigned flags, cups_dest_t *dest)
{
   EnumDestsContext *context = (EnumDestsContext *) user_data;
   if ((flags & (CUPS_DEST_FLAGS_UNCONNECTED |
                 CUPS_DEST_FLAGS_REMOVED |
                 CUPS_DEST_FLAGS_ERROR |
                 CUPS_DEST_FLAGS_RESOLVING |
                 CUPS_DEST_FLAGS_CONNECTING |
                 CUPS_DEST_FLAGS_CANCELED)) == 0) {

       context->num_printers = cupsCopyDest (dest, context->num_printers,
                                              &context->printers);

       // Also copy whether this is the local default /
       cups_dest_t *the_dest;
       the_dest = cupsGetDest(dest->name, dest->instance,
                               context->num_printers, context->printers);
       the_dest->is_default = dest->is_default;
       qDebug() << dest->name << "_" << dest->instance << "_" << context->num_printers;
   }
   return 1;
}

QT_BEGIN_NAMESPACE

QCupsPrinterSupport::QCupsPrinterSupport()
    : QPlatformPrinterSupport()
{
}

QCupsPrinterSupport::~QCupsPrinterSupport()
{
}

QPrintEngine *QCupsPrinterSupport::createNativePrintEngine(QPrinter::PrinterMode printerMode, const QString &deviceId)
{
    return new QCupsPrintEngine(printerMode, (deviceId.isEmpty() ? defaultPrintDeviceId() : deviceId));
}

QPaintEngine *QCupsPrinterSupport::createPaintEngine(QPrintEngine *engine, QPrinter::PrinterMode printerMode)
{
    Q_UNUSED(printerMode)
    return static_cast<QCupsPrintEngine *>(engine);
}

QPrintDevice QCupsPrinterSupport::createPrintDevice(const QString &id)
{
    return QPlatformPrinterSupport::createPrintDevice(new QPpdPrintDevice(id));
}

QStringList QCupsPrinterSupport::availablePrintDeviceIds() const
{
/*  // Reset cache disabled for now
    if (qt_cups_printers) {
        cupsFreeDests( qt_cups_num_printers, qt_cups_printers );
        qt_cups_printers = NULL;
        qt_cups_num_printers = 0;
    }
*/
    cups_dest_t *dests;
    bool enablePrinterDiscovery = qEnvironmentVariableIsSet("QT_ENABLE_PRINTER_DISCOVERY");
    if (!enablePrinterDiscovery) {
        qt_cups_num_printers = cupsGetDests(&dests);
    } else {
        if (qt_cups_num_printers == 0)
            QCupsPrinterSupport::fillCupsPrinters();

        dests = qt_cups_printers;
    }
    QStringList list;
    list.reserve(qt_cups_num_printers);
    for (int i = 0; i < qt_cups_num_printers; ++i) {
        QString printerId = QString::fromLocal8Bit(dests[i].name);
        if (dests[i].instance)
            printerId += QLatin1Char('/') + QString::fromLocal8Bit(dests[i].instance);
        list.append(printerId);
    }
    if (!enablePrinterDiscovery) {
        cupsFreeDests(qt_cups_num_printers, dests);
    }

    return list;
}

QString QCupsPrinterSupport::defaultPrintDeviceId() const
{
    QString printerId;
    cups_dest_t *dests;

    bool enablePrinterDiscovery = qEnvironmentVariableIsSet("QT_ENABLE_PRINTER_DISCOVERY");
    if (!enablePrinterDiscovery) {
        qt_cups_num_printers = cupsGetDests(&dests);
    } else {
        if (qt_cups_num_printers == 0)
            QCupsPrinterSupport::fillCupsPrinters();

        dests = qt_cups_printers;
    }

    for (int i = 0; i < qt_cups_num_printers; ++i) {
        if (dests[i].is_default) {
            printerId = QString::fromLocal8Bit(dests[i].name);
            if (dests[i].instance) {
                printerId += QLatin1Char('/') + QString::fromLocal8Bit(dests[i].instance);
                break;
            }
        }
    }
    if (!enablePrinterDiscovery) {
        cupsFreeDests(qt_cups_num_printers, dests);
    }
    return printerId;
}

void QCupsPrinterSupport::fillCupsPrinters()
{
    EnumDestsContext context;
    context.printers = 0;
    context.num_printers = 0;

    qDebug() << "begin enumerating printers";

    cupsEnumDests(CUPS_DEST_FLAGS_NONE, 4000, NULL, 0, 0,
                   enum_dest_cb2, &context);

    qDebug() << "end enumerating printers";
    qt_cups_printers = context.printers;
    qt_cups_num_printers = context.num_printers;
}

cups_dest_t *QCupsPrinterSupport::qt_cups_printers = NULL;
int QCupsPrinterSupport::qt_cups_num_printers = 0;


QT_END_NAMESPACE
